package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.core.solar.config.SolarProperties;
import com.qmth.boot.core.solar.model.AppInfo;
import com.qmth.boot.core.solar.model.OrgInfo;
import com.qmth.boot.core.solar.service.SolarService;
import com.qmth.distributed.print.business.entity.TSAuth;
import com.qmth.distributed.print.business.enums.AuthEnum;
import com.qmth.distributed.print.business.service.AuthInfoService;
import com.qmth.distributed.print.business.service.TSAuthService;
import com.qmth.teachcloud.common.bean.dto.AuthOrgInfoDto;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.CommonCacheService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

/**
 * @Description: 授权信息service
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2022/5/17
 */
@Service
public class AuthInfoServiceImpl implements AuthInfoService {
    private final static Logger log = LoggerFactory.getLogger(AuthInfoServiceImpl.class);

    @Resource
    SolarProperties solarProperties;

    @Resource
    SolarService solarService;

    @Resource
    TSAuthService tsAuthService;

    @Resource
    CommonCacheService commonCacheService;

    /**
     * 授权信息初始化
     *
     * @return
     */
    @Override
    public AppInfo appInfoInit() {
        AppInfo appInfo = solarService.getAppInfo();
        if (Objects.nonNull(appInfo) && Objects.nonNull(solarProperties)) {
            if (Objects.nonNull(solarProperties.getAccessKey())
                    && !Objects.equals(solarProperties.getAccessKey().trim(), "")
                    && Objects.nonNull(solarProperties.getAccessSecret())
                    && !Objects.equals(solarProperties.getAccessSecret().trim(), "")) {//在线激活
                saveAuthInfo(appInfo, AuthEnum.ON_LINE);
            } else if (Objects.nonNull(solarProperties.getLicense())
                    && !Objects.equals(solarProperties.getLicense().trim(), "")) {//离线激活
                saveAuthInfo(appInfo, AuthEnum.OFF_LINE);
            }
        }
        return appInfo;
    }

    /**
     * app是否过期
     *
     * @param schoolId
     * @return
     */
    @Override
    public void appHasExpired(Long schoolId) {
        if (Objects.nonNull(schoolId)) {
            AuthOrgInfoDto authOrgInfoDto = commonCacheService.authInfoCache(schoolId);
            if (Objects.isNull(authOrgInfoDto) || (Objects.nonNull(authOrgInfoDto) && authOrgInfoDto.getControl().hasExpired())) {
                throw ExceptionResultEnum.AUTH_INFO_ERROR.exception();
            }
        }
    }

    /**
     * 保存鉴权信息
     *
     * @param appInfo
     * @param authEnum
     */
    @Transactional
    protected void saveAuthInfo(AppInfo appInfo, AuthEnum authEnum) {
        List<OrgInfo> orgInfoList = solarService.getOrgList();
        List<TSAuth> tsAuthList = new ArrayList<>();
        Set<Long> orgIdsSet = new HashSet<>();
        for (OrgInfo o : orgInfoList) {
            orgIdsSet.add(o.getId());
            if (authEnum == AuthEnum.OFF_LINE) {
                tsAuthList.add(new TSAuth(o.getId(), solarProperties.getLicense(), authEnum, appInfo.getControl().getExpireTime()));
            } else {
                tsAuthList.add(new TSAuth(o.getId(), solarProperties.getAccessKey(), solarProperties.getAccessSecret(), authEnum, appInfo.getControl().getExpireTime()));
            }
            commonCacheService.updateAuthInfoCache(o.getId());
        }
        QueryWrapper<TSAuth> tsAuthQueryWrapper = new QueryWrapper<>();
        tsAuthQueryWrapper.lambda().in(TSAuth::getSchoolId, orgIdsSet);
        tsAuthService.remove(tsAuthQueryWrapper);
        tsAuthService.saveOrUpdateBatch(tsAuthList);
    }
}
